/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Bookmarks Menu Button.
 *
 * The Initial Developer of the Original Code is Ronny Perinke.
 * Portions created by the Initial Developer are Copyright (C) 2004-2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 *   Ronny Perinke <www.sephiroth-j.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

const bmbext = {
	//#######################################
	//########	private functions and properties	########
	//########						########
	//#######################################
	_Prefs:
	{
		hideBM: null,				// value for hiding the bookmarks menu
		hideBMBS: null,				// value for hiding my BMB settings menu entry
		hideDropmarker: false,			// hide the dropmarker?
		MiddleClickOpens: 1,			// middle-click on button loads ...
		RightClickOpens: 2,			// right-click on button loads ...
		firstRun: null				// is it the first time that the ext is used?
	},
	_MiscVars:
	{
		MenuIsThere: false,			//
		LastOpenFolders: null,			// holds the array for the menus needed to open
		DefaultBMEntryCount: 3,		// the number of menuentries of the default bookmarks-menu without the bookmarks. by default it should be 3 for fx 1.5 and 1.0 (see browser.xul).
		ShortcutKey: null,
		ShortcutKeyPressed: false,		// was the shortcut key pressed?
		StringBundle: null,			// holds the stringbundleset
		PrefsIFace: null,				// refers to the nsIPref interface
		StatusBar: null,				// refers to the browsers statusbar object
		bmbCSSSheet:				// Container for the CSS-Style sheet of BMB
		{
			globalIndex: -1,			// its index in documents.styleSheets
			sheetObj: null,			// the styleSheet object itself
			dropmarkerHidden: false	// did we hide the dropmarker?
		}
	},
	get prefRoot()
	{
		return "BMBExt.";
	},
	set prefRoot(aString){},
	get StringBundle()
	{
		if (!this._MiscVars.StringBundle)
		{
			this.StringBundle = "";
		}
		return this._MiscVars.StringBundle;
	},
	set StringBundle(aBundle)
	{
		this._MiscVars.StringBundle = document.getElementById(!aBundle?"bmbextstr":aBundle);
	},
	get PrefsIFace()
	{
		if (!this._MiscVars.PrefsIFace)
		{
			this.PrefsIFace = this.prefRoot;
		}
		return this._MiscVars.PrefsIFace;
	},
	set PrefsIFace(aBranch)
	{
		if (typeof(aBranch) != "string")
		{
			throw Components.results.NS_ERROR_ILLEGAL_VALUE + "\nValue must be a string type";
		}
		
		this._MiscVars.PrefsIFace = Components.classes["@mozilla.org/preferences;1"].getService(Components.interfaces.nsIPref);
		this._MiscVars.PrefsIFace = this._MiscVars.PrefsIFace.getBranch(aBranch?aBranch:this.prefRoot);
	},
	get StatusBarObj()
	{
		if (!this._MiscVars.StatusBar)
		{
			this.StatusBarObj = "";
		}
		return this._MiscVars.StatusBar;
	},
	set StatusBarObj()
	{
		this._MiscVars.StatusBar = document.getElementById("statusbar-display");
	},
	get BMBCSSSheet()
	{
		return this._MiscVars.bmbCSSSheet;
	},
	//#######################################
	//########	/private functions and properties	########
	//########						########
	//#######################################
	
	init: function()
	{
		bmbext.addBMBsupport();
		bmbextSort.init();
		bmbext.loadPrefs();
		bmbext.deleteOldPrefs();
		bmbext.findBMBCSSSheet();
		bmbext.showHideDropmarker();
		setTimeout("bmbext.addButtonAtFirstTime()", 1000);		//add menubutton to nav-bar on first startup after 1sec
		bmbext.showHide();
		bmbext.setBehavior(false);
		
		var menupopup = document.getElementById('menu_BookmarksPopup');
		try
		{
			bmbext._MiscVars.DefaultBMEntryCount = menupopup.childNodes.length;
		}
		catch (e)
		{
			bmbext._MiscVars.DefaultBMEntryCount = 4;
		}
		menupopup.lastChild.id = "LastSepBeforeBMs";
		
		try
		{
			if (document.getElementById('bookmarks-menu-button'))
			{
				bmbext._MiscVars.ShortcutKey = document.getElementById('bookmarks-menu').getAttribute("accesskey").charCodeAt(0);		//converts the character to the ASCII code because keyCode is a ASCII code
				window.addEventListener('keydown', bmbext.catchKKey, false);
				window.addEventListener('keyup', bmbext.catchKKey, false);
			}
		}
		catch (e)
		{
			alert("BMB Extension\n\nCould not add EventListener #1!\nError Msg is:\n" + e);
		}
		
		try
		{
			var templ = document.getElementById("bookmarks-template");	//get the bookmarks-tree-template
			templ = templ.lastChild;							//the last rule is the one for the bookmarks
			templ.firstChild.firstChild.setAttribute("tooltiptext", "rdf:http://home.netscape.com/NC-rdf#Name");	//add the bookmarks label as tooltip for them by changing a specific rule
			templ.firstChild.firstChild.setAttribute("onmouseover", "bmbext.setStatus(this);");	//updating the statusbar with the url of the bookmark
		}
		catch (e)
		{}
		
		bmbextPrefObserver.register(bmbext.prefRoot);
	},
	quit: function()
	{
		bmbextPrefObserver.unregister();
	},
	
	deleteOldPrefs: function()	//delete old, unused preferences, if set
	{
		if (!this._Prefs.firstRun)
		{
			return;
		}
		
		this.PrefsIFace = this.prefRoot;
		
		try
		{
			this.PrefsIFace.getBoolPref("settings.allowSidebarOnMClk");
			this.PrefsIFace.clearUserPref("settings.allowSidebarOnMClk");
		}
		catch (e)
		{}
		try
		{
			this.PrefsIFace.getBoolPref("settings.allowManagerOnRClk");
			this.PrefsIFace.clearUserPref("settings.allowManagerOnRClk");
		}
		catch (e)
		{}
		// new in 0.4.2; because it was a 1.0-only option/feature
		try
		{
			this.PrefsIFace.getBoolPref("settings.openLastUsedFolder");
			this.PrefsIFace.clearUserPref("settings.openLastUsedFolder");
		}
		catch (e)
		{}
	},
	loadPrefs: function()
	{
		//load user settings
		this.PrefsIFace = this.prefRoot;	// init PrefsIFace with default pref-branch
		
		this._Prefs.hideBM = this.PrefsIFace.getBoolPref("bookmarkmenu.hide");
		this._Prefs.hideBMBS = this.PrefsIFace.getBoolPref("settings.hide");
		this._Prefs.hideDropmarker = this.PrefsIFace.getBoolPref("dropmarker.hide");
		this._Prefs.firstRun = this.PrefsIFace.getBoolPref("firstRun");
		this._Prefs.MiddleClickOpens = this.PrefsIFace.getIntPref("settings.MiddleClickOpens");
		this._Prefs.RightClickOpens = this.PrefsIFace.getIntPref("settings.RightClickOpens");
	},
	
	addButtonAtFirstTime: function()
	{
		//add menubutton to nav-bar on first startup and search in all custom toolbars if BMB is in there
		var navbar = document.getElementById("nav-bar");
		var toolbars = document.getElementsByTagName("toolbar");
		if (this._Prefs.firstRun)
		{
			var foundMB = false;
			var currentset = "";
			var erg;
			for (var i= 0, l=toolbars.length; i < l; ++i)
			{
				if (toolbars[i].getAttribute("class").indexOf("chromeclass") != -1)
				{
					currentset = toolbars[i].currentSet;
					erg = currentset.search(/(bookmarks|history)-menu-button/g);
					if (erg != -1)
					{
						foundMB = true;
						break;
					}
				}
			}
			toolbars = null;
			
			if (!foundMB && !navbar.getAttribute("collapsed"))
			{
				check = confirm(this.StringBundle.getString("bmb-ext.FirstUsageAddButton"));
				if (check)
				{
					var menu = document.getElementById('bookmarks-menu');
					menu.hidden = true;
					var aBeforeElt=document.getElementById("urlbar-container");
					navbar.insertItem("bookmarks-menu-button", aBeforeElt);
					navbar.insertItem("history-menu-button", aBeforeElt);
					currentset = navbar.currentSet;
					navbar.setAttribute("currentset", currentset);
					document.persist("nav-bar","currentset");
					
					var menupopup = document.getElementById('menu_BookmarksPopup');
					var childnodes = menupopup.childNodes;
					for (var i=0, l=childnodes.length; i<l; i++)
					{
						if (childnodes[i].id == "bmb-settings")
						{
							menupopup.removeChild(childnodes[i]);
							break;
						}
					}
				}
			}
		}
		this._Prefs.firstRun = false;
		this.PrefsIFace = "";
		this.PrefsIFace.setBoolPref("firstRun", false);
	},
	
	showHide: function()
	{
		var menu = document.getElementById('bookmarks-menu');
		var bmb = document.getElementById('bookmarks-menu-button');
		if (bmb)
		{
			if (this._Prefs.RightClickOpens == 2 || this._Prefs.RightClickOpens == 1)
			{
				bmb.setAttribute("context","");
			}
			else
			{
				bmb.setAttribute("context", "toolbar-context-menu");
			}
			if (menu)
			{
				menu.hidden = this._Prefs.hideBM;
			}
		}
		else
		{
			//add the BMB settings to the normal bookmarks menu, if not already added
			var bmbSettings = document.getElementById("bmb-settings");
			if (menu && !menu.hidden && !bmbSettings)
			{
				bmbSettings = document.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "menuitem");
				bmbSettings.id = "bmb-settings";
				bmbSettings.setAttribute("label", this.StringBundle.getString("bmb-ext.menu-label"));
				bmbSettings.setAttribute("tooltiptext", this.StringBundle.getString("bmb-ext.bmb-tooltip1"));
				bmbSettings.setAttribute("oncommand", "bmbext.OpenSettings(0)");
				bmbSettings.setAttribute("class", "menuitem-iconic");
				menu.firstChild.insertBefore(bmbSettings, menu.firstChild.firstChild);
			}
			bmbSettings = null;
		}
		bmb = null; menu = null;
	},
	findBMBCSSSheet: function()
	{
		var CSSSheets = document.styleSheets;
		for (var i=0, l=CSSSheets.length; i<l; i++)
		{
			if (CSSSheets[i].href == "chrome://bmb-ext/skin/bmb-ext.css")
			{
				this.BMBCSSSheet.globalIndex = i;
				this.BMBCSSSheet.sheetObj = CSSSheets[i];
				CSSSheets = null;
				break;
			}
		}
	},
	showHideDropmarker: function()
	{
		var sheet = this.BMBCSSSheet;
		if (sheet.globalIndex < 0)	// was not found
			return;
		
		var lastRuleIndex = sheet.sheetObj.cssRules.length - 1;
		
		if (!sheet.dropmarkerHidden && this._Prefs.hideDropmarker)
		{
			sheet.sheetObj.insertRule("#bookmarks-menu-button .toolbarbutton-menu-dropmarker { display: none !important;}", lastRuleIndex + 1);
			sheet.dropmarkerHidden = true;
			return;
		}
		
		if (sheet.dropmarkerHidden && !this._Prefs.hideDropmarker)
		{
			var indexToRemove = -1;
			for (var i=lastRuleIndex; i>=0; i--)
			{
				if (sheet.sheetObj.cssRules[i].selectorText == "#bookmarks-menu-button .toolbarbutton-menu-dropmarker")
				{
					indexToRemove = i;
					break;
				}
			}
			
			if (indexToRemove >= 0)
			{
				sheet.sheetObj.deleteRule(indexToRemove);
				sheet.dropmarkerHidden = false;
			}
		}
	},
	
	buildMenu: function()
	{
		var menupopup = document.getElementById('menu_BookmarksPopup');
		var bmbSettings = document.getElementById('bmb-settings');
		//remove possible added BMB settings by showHide(), when BMB was not present
		if (bmbSettings && bmbSettings.parentNode.id == menupopup.id)
		{
			menupopup.removeChild(bmbSettings);
		}
		bmbSettings = null;
		
		var bmbmenu = document.getElementById('menubutton_BookmarksPopup');
		var bmbmenu_bmsep = document.getElementById('bmb-LastSepBeforeBMs');
		
		this._MiscVars.DefaultBMEntryCount = menupopup.childNodes.length;
		var clone;
		if (!bmbmenu.hasChildNodes())
		{
			for (var i=0, l=this._MiscVars.DefaultBMEntryCount; i<l; i++)
			{
				clone = menupopup.childNodes[i].cloneNode(true);
				//workaround for sortbookmarks extension v0.6
				if (clone.id == "bookmarks-menuitem-sort" && typeof(sortBookmarksService) == "object")
				{
					clone.label = sortBookmarksService.menuItemLabel;
				}
				// end of workaround
				if (clone.id == "bookmarkAllCmd" || clone.id == "subscribeToPageMenuitem" || clone.id == "subscribeToPageMenupopup")
				{
					clone.id = "bmb-" + clone.id;
				}
				if (clone.id == "LastSepBeforeBMs")
				{
					break;
				}
				bmbmenu.insertBefore(clone, bmbmenu_bmsep);
			}
		}
		else
		{
			const aBeforeElt = bmbmenu.firstChild;
			for (var i=0, l=this._MiscVars.DefaultBMEntryCount; i<l; i++)
			{
				clone = menupopup.childNodes[i].cloneNode(true);
				//workaround for sortbookmarks extension v0.6
				if (clone.id == "bookmarks-menuitem-sort" && typeof(sortBookmarksService) == "object")
				{
					clone.label = sortBookmarksService.menuItemLabel;
				}
				if (clone.id == "bookmarkAllCmd" || clone.id == "subscribeToPageMenuitem" || clone.id == "subscribeToPageMenupopup")
				{
					clone.id = "bmb-" + clone.id;
				}
				if (clone.id == "LastSepBeforeBMs")
				{
					break;
				}
				bmbmenu.insertBefore(clone, aBeforeElt);
			}
		}
		clone = null;
		
		var item, itemCommand;
		for (var i=0, l=this._MiscVars.DefaultBMEntryCount; i<l; i++)
		{
			item = bmbmenu.childNodes[i];
			if (item.id == "bmb-LastSepBeforeBMs")
			{
				break;
			}
			
			if (item.getAttribute("command") != "")
			{
				itemCommand = item.getAttribute("command");
			}
			else
			{
				itemCommand = item.getAttribute("oncommand");
			}
			
			if (itemCommand.search(/Browser:AddBookmarkAs/gi) != -1)
			{
				item.tooltipText = this.StringBundle.getString("bmb-ext.bmb-tooltip2");
				item.className = "menuitem-iconic bmadd";
			}
			if (itemCommand.search(/'bookmarks:manager'/gi) != -1) {
				item.tooltipText = this.StringBundle.getString("bmb-ext.bmb-tooltip3");
				item.className = "menuitem-iconic bmmanage";
			}
			if (item.id == "bmb-bookmarkAllCmd" || itemCommand.search(/Browser:BookmarkAllTabs/gi) != -1)
			{
				item.tooltipText = this.StringBundle.getString("bmb-ext.bmb-tooltip4");
				item.className = "menuitem-iconic tabadd";
			}
			if (item.id == "bmb-subscribeToPageMenuitem")
			{
				item.className = "menuitem-iconic feedadd";
			}
			if (item.id == "bmb-subscribeToPageMenupopup")
			{
				item.className = "menu-iconic feedadd";
				item.firstChild.removeAttribute("id");
				item.firstChild.setAttribute("onpopupshown", "bmbext.updateSubscribeToPageMenuitems();");
			}
		}
		item = null; itemCommand = null;
		
		//add 'toggle sidebar' menu entry
		//get it and clone it from the viewSidebarMenu
		var sidebarMenu = document.getElementById('viewSidebarMenu');
		var sidebar_entry_clone = sidebarMenu.getElementsByTagName("menuitem")[0].cloneNode(false);
		sidebar_entry_clone.tooltipText = this.StringBundle.getString("bookmarksButton.tooltip");
		sidebar_entry_clone.className = "menuitem-iconic bmsidebar";
		bmbmenu.insertBefore(sidebar_entry_clone, bmbmenu_bmsep);	//and place it before this sep.
		sidebarMenu = null; sidebar_entry_clone = null;
		
		//add 'bmb settings' if wanted
		if (!this._Prefs.hideBMBS)
		{
			bmbSettings = document.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "menuitem");
			bmbSettings.id = "bmb-settings";
			bmbSettings.setAttribute("label", this.StringBundle.getString("bmb-ext.menu-label"));
			bmbSettings.setAttribute("tooltiptext", this.StringBundle.getString("bmb-ext.bmb-tooltip1"));
			bmbSettings.setAttribute("oncommand", "bmbext.OpenSettings(0)");
			bmbSettings.setAttribute("class", "menuitem-iconic");
			var mysep = document.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "menuseparator");
			mysep.id = "bmb-settings-sep";
			bmbmenu.insertBefore(mysep, bmbmenu.firstChild);
			bmbmenu.insertBefore(bmbSettings, mysep);
			mysep = null; bmbSettings = null;
		}
		
		this._MiscVars.MenuIsThere = true;
	},
	destroyMenu: function()
	{
		var bmb = document.getElementById("bookmarks-menu-button");
		var bmbmenu = document.getElementById("menubutton_BookmarksPopup");
		if (!bmb.open)
		{
			if (bmbmenu)
			{
				for (var i=0, l=bmbmenu.childNodes.length; i<l; i++)
				{
					bmbmenu.removeChild(bmbmenu.firstChild);
					if (bmbmenu.firstChild.id == "bmb-LastSepBeforeBMs")
					{
						break;
					}
				}
			}
			this.setBehavior(false);
			this._MiscVars.MenuIsThere = false;
		}
	},
	showMenu: function()
	{
		if (!this._MiscVars.MenuIsThere)
		{
			this.setBehavior(true);
			this.buildMenu();
		}
		else
		{
			this.updateSubscribeToPage();
		}
	},
	
	UpdateBookmarkAllTabsMenuitem: function()
	{
		var bookmarkAllCmd = document.getElementById('bmb-bookmarkAllCmd');
		if (bookmarkAllCmd)
		{
			UpdateBookmarkAllTabsMenuitem();
		}
	},
	updateSubscribeToPage: function()
	{
		var feedButton = document.getElementById("feed-button");
		if (!feedButton)
			return;
			
		var feedMenuitem = document.getElementById("bmb-subscribeToPageMenuitem");
		if (!feedMenuitem)
			return;
			
		var feedMenupopup = document.getElementById("bmb-subscribeToPageMenupopup");
		if (!feedMenupopup)
			return;
		
		var feeds = gBrowser.mCurrentBrowser.feeds;
		if (!feeds || feeds.length == 0)
		{
			feedMenupopup.setAttribute("hidden", "true");
			feedMenuitem.setAttribute("disabled", "true");
			feedMenuitem.removeAttribute("hidden");
		}
		else
		{
			feeds = FeedHandler.harvestFeeds(feeds);		// remove possible double entries
			if (feeds.length > 1)
			{
				feedMenupopup.removeAttribute("hidden");
				feedMenuitem.setAttribute("hidden", "true");
			}
			else
			{
				feedMenupopup.setAttribute("hidden", "true");
				feedMenuitem.setAttribute("feed", feeds[0].href);
				feedMenuitem.removeAttribute("disabled");
				feedMenuitem.removeAttribute("hidden");
			}
		}
	},
	updateSubscribeToPageMenuitems: function()
	{
		var feedMenupopup = document.getElementById("bmb-subscribeToPageMenupopup");
		var items = feedMenupopup.getElementsByTagName("menuitem");
		for (var i=0, l=items.length; i < l; i++)
		{
			items[i].className = "menuitem-iconic feed";
		}
	},
	
	setStatus: function(aTarget)	//sets the text of the statusbar
	{
		this.StatusBarObj.label = aTarget.statusText;
	},
	unsetStatus: function()	//clears the statusbar text again
	{
		this.StatusBarObj.removeAttribute('label');
	},
	
	catchMKey: function(aEvent, aDS)
	{
		if (aEvent.currentTarget.disabled)	//aEvent.currentTarget is bmb
			return;
		
		switch (aEvent.button)
		{
			case 0:	//left button uses oncommand
				break;
				
			case 1:	//middle button
				if (aEvent.currentTarget.open)
				{
					BookmarksMenu.loadBookmarkMiddleClick(aEvent, aDS);
				}
				else
				{
					switch (this._Prefs.MiddleClickOpens)
					{
						case 0:
							break;
						case 1:
							toggleSidebar('viewBookmarksSidebar');
							break;
						case 2:
							toOpenWindowByType('bookmarks:manager', 'chrome://browser/content/bookmarks/bookmarksManager.xul');
							break;
						default:
							break;
					}
				}
				break;
				
			case 2:	//right button
				if (!aEvent.currentTarget.open)
				{
					switch (this._Prefs.RightClickOpens)
					{
						case 0:
							break;
						case 1:
							toggleSidebar('viewBookmarksSidebar');
							break;
						case 2:
							toOpenWindowByType('bookmarks:manager', 'chrome://browser/content/bookmarks/bookmarksManager.xul');
							break;
						default:
							break;
					}
				}	
				break;
				
			default:	//unknown
				break;
		}
	},
	
	openMenu: function()
	{
		this.showMenu();	//just in case the button was not hoverd before so i need to build the menu
		var bmb = document.getElementById('bookmarks-menu-button');	
		document.getElementById('menubutton_BookmarksPopup').showPopup(bmb, -1, -1, "popup");
	},
	
	catchKKey: function(aEvent)	//shortcut key event handler
	{
		var bmb = document.getElementById('bookmarks-menu-button');
		if (!bmb || bmb.open)
		{
			bmbext._MiscVars.ShortcutKeyPressed = false;
			return;
		}
		
		if (!bmbext._MiscVars.ShortcutKey)
		{
			return;
		}
		
		if (aEvent.type== 'keydown' && !bmbext._MiscVars.ShortcutKeyPressed)
		{
			if (aEvent.altKey && (aEvent.keyCode == bmbext._MiscVars.ShortcutKey))
			{
				bmbext._MiscVars.ShortcutKeyPressed = true;
			}
		}
		if (aEvent.type== 'keyup' && bmbext._MiscVars.ShortcutKeyPressed)
		{
			if (aEvent.altKey && (aEvent.keyCode == bmbext._MiscVars.ShortcutKey))
			{
				bmbext._MiscVars.ShortcutKeyPressed = false;
				bmbext.openMenu();
			}
		}
	},
	
	setBehavior: function(menu)
	{
		//this function is used to switch between the two different behaviors
		//when closed you can drag and drop a link on the button to add it to your bookmarks
		//when open you can move bookmarks and folders within the menu
		var bmb = document.getElementById('bookmarks-menu-button');	
		if (bmb)
		{
			if (menu)
			{
				bmb.setAttribute("ondraggesture", "nsDragAndDrop.startDrag(event, BookmarksMenuDNDObserver)");
				bmb.setAttribute("ondragdrop", "nsDragAndDrop.drop(event, BookmarksMenuDNDObserver); event.preventBubble()");
				bmb.setAttribute("ondragenter", "nsDragAndDrop.dragEnter(event, BookmarksMenuDNDObserver); event.preventBubble()");
				bmb.setAttribute("ondragexit", "nsDragAndDrop.dragExit(event, BookmarksMenuDNDObserver); event.preventBubble()");
				bmb.setAttribute("ondragover", "nsDragAndDrop.dragOver(event, BookmarksMenuDNDObserver); event.preventBubble()");
			}
			else
			{
				bmb.removeAttribute("ondraggesture");
				bmb.setAttribute("ondragdrop", "nsDragAndDrop.drop(event, bookmarksButtonObserver); event.preventBubble()");
				bmb.removeAttribute("ondragenter");
				bmb.setAttribute("ondragexit", "nsDragAndDrop.dragExit(event, bookmarksButtonObserver); event.preventBubble()");
				bmb.setAttribute("ondragover", "nsDragAndDrop.dragOver(event, bookmarksButtonObserver); event.preventBubble()");
			}
		}
	},
	
	OpenSettings: function(aTabIndex)
	{
		if (aTabIndex)
		{
			window.openDialog('chrome://bmb-ext/content/bmb-ext_prefDialog.xul', 'wBMB_Settings', 'chrome,centerscreen,resizable', aTabIndex);
		}
		else
		{
			window.openDialog('chrome://bmb-ext/content/bmb-ext_prefDialog.xul', 'wBMB_Settings', 'chrome,centerscreen,resizable', 0);
		}
	},
	
	addBMBsupport: function() {
		eval('BookmarksMenu.getBTSelection='+BookmarksMenu.getBTSelection.toString().replace(
			'case "bookmarks-menu":',
			'case "bookmarks-menu": \
			case "bookmarks-menu-button":')
		);
		eval('BookmarksMenu.getBTTarget='+BookmarksMenu.getBTTarget.toString().replace(
			'case "bookmarks-menu":',
			'case "bookmarks-menu": \
			case "bookmarks-menu-button":')
		);
		eval('BookmarksMenu.getBTContainer='+BookmarksMenu.getBTContainer.toString().replace(
			'case "bookmarks-menu":',
			'case "bookmarks-menu": \
			case "bookmarks-menu-button":')
		);
		eval('BookmarksMenuDNDObserver.onDragStart='+BookmarksMenuDNDObserver.onDragStart.toString().replace(
			'target.id == "bookmarks-menu"',
			'target.id == "bookmarks-menu" || \
			target.id == "bookmarks-menu-button"')
		);
		eval('BookmarksMenuDNDObserver.canDrop='+BookmarksMenuDNDObserver.canDrop.toString().replace(
			'target.id == "bookmarks-menu"',
			'target.id == "bookmarks-menu" || \
			target.id == "bookmarks-menu-button"')
		);
		//
		BookmarksMenuDNDObserver.__defineGetter__('mObservers', function ()
		{
			if (!this._observers || this._observers.length < 4)
			{
				var bmb = document.getElementById("bookmarks-menu-button");
				if (bmb)
				{
					if (bmb.firstChild)
					{
						this._observers = [
							document.getElementById("bookmarks-ptf"),
							document.getElementById("bookmarks-menu").firstChild,
							bmb.firstChild,
							document.getElementById("bookmarks-chevron").parentNode
						]
					}
					else
					{
						this._observers = [
							document.getElementById("bookmarks-ptf"),
							document.getElementById("bookmarks-menu").firstChild,
							document.getElementById("bookmarks-chevron").parentNode
						]
					}
				}
				else
				{
					this._observers = [
						document.getElementById("bookmarks-ptf"),
						document.getElementById("bookmarks-menu").firstChild,
						document.getElementById("bookmarks-chevron").parentNode
					]
				}
			}
			return this._observers;
		});
		//
		eval('BookmarksMenuDNDObserver.onDragCloseTarget='+BookmarksMenuDNDObserver.onDragCloseTarget.toString().replace(
			'if (this.mObservers[i].parentNode.id == "bookmarks-menu")',
			'if (this.mObservers[i].parentNode.id == "bookmarks-menu" || this.mObservers[i].parentNode.id == "bookmarks-menu-button")')
		);
	}
}

window.addEventListener('load', bmbext.init, false);		//extension should load on browser start-up
window.addEventListener('unload', bmbext.quit, false);	//and unload on exit