/* ***** BEGIN LICENSE BLOCK *****
* Version: MPL 1.1/GPL 2.0/LGPL 2.1
*
* The contents of this file are subject to the Mozilla Public License Version
* 1.1 (the "License"); you may not use this file except in compliance with
* the License. You may obtain a copy of the License at
* http://www.mozilla.org/MPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
* for the specific language governing rights and limitations under the
* License.
*
* The Original Code is Bookmarks Menu Button.
*
* The Initial Developer of the Original Code is Ronny Perinke.
* Portions created by the Initial Developer are Copyright (C) 2004-2008
* the Initial Developer. All Rights Reserved.
*
* Contributor(s):
*
*   Ronny Perinke <www.sephiroth-j.de>
*
* Alternatively, the contents of this file may be used under the terms of
* either the GNU General Public License Version 2 or later (the "GPL"), or
* the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
* in which case the provisions of the GPL or the LGPL are applicable instead
* of those above. If you wish to allow use of your version of this file only
* under the terms of either the GPL or the LGPL, and not to allow others to
* use your version of this file under the terms of the MPL, indicate your
* decision by deleting the provisions above and replace them with the notice
* and other provisions required by the GPL or the LGPL. If you do not delete
* the provisions above, a recipient may use your version of this file under
* the terms of any one of the MPL, the GPL or the LGPL.
*
* ***** END LICENSE BLOCK ***** */

var bmbextHistory = {
 	//#######################################
	//########	private functions and properties	########
	//########						########
	//#######################################
	_Prefs:
	{
		hideDropmarker: false,				// hide the dropmarker?
		hideGoMenu: false,				// hide normal history menu?
		hideBMBS: false,					// remove BMB-Settings from history button?
		MiddleClickOpens: 1,				// middle-click on button loads ...
		RightClickOpens: 1,				// right-click on button loads ...
		GroupMethod: 2,					// how to group the history
		MaxRecentHistory: 10				// max recently visited sites
	},
	_MiscVars:
	{
		ShortcutKey: null,
		ShortcutKeyPressed: false,			// was the shortcut key pressed?
		Selection: null,
		HistoryNeedsRebuild: false,			// sometimes the history menu is not updated correctly so i need to force it to do so
		StringBundle: null,				// holds my stringbundleset
		PrefsIFace: null,					// refers to the nsIPref interface
		bmbCSSSheet:					// container for the CSS-Style sheet of BMB
		{
			globalIndex: -1,				// its index in documents.styleSheets
			sheetObj: null,				// the styleSheet object itself
			dropmarkerHidden: false		// did we hide the dropmarker?
		},
		canUndoCloseTab: (typeof(undoCloseTab) == "function")
	},	
	get prefRoot()
	{
		return "BMBExt.History.";
	},
	set prefRoot(aString){},
	get StringBundle()
	{
		if (!this._MiscVars.StringBundle)
		{
			this.StringBundle = "";
		}
		return this._MiscVars.StringBundle;
	},
	set StringBundle(aBundle)
	{
		this._MiscVars.StringBundle = document.getElementById(!aBundle?"bmbextstr":aBundle);
	},
	get PrefsIFace()
	{
		if (!this._MiscVars.PrefsIFace)
		{
			this.PrefsIFace = this.prefRoot;
		}
		return this._MiscVars.PrefsIFace;
	},
	set PrefsIFace(aBranch)
	{
		if (typeof(aBranch) != "string")
		{
			throw Components.results.NS_ERROR_ILLEGAL_VALUE + "\nValue must be a string type";
		}
		
		this._MiscVars.PrefsIFace = Components.classes["@mozilla.org/preferences;1"].getService(Components.interfaces.nsIPref);
		this._MiscVars.PrefsIFace = this._MiscVars.PrefsIFace.getBranch(aBranch?aBranch:this.prefRoot);
	},
	get HistoryNeedsRebuild()
	{
		return this._MiscVars.HistoryNeedsRebuild;
	},
	set HistoryNeedsRebuild(aValue)
	{
		if (typeof(aValue) == "boolean")
		{
			this._MiscVars.HistoryNeedsRebuild = aValue;
		}
		else
		{
			throw Components.results.NS_ERROR_ILLEGAL_VALUE + "\nValue must be a boolean type";
		}
	},
	get Selection()
	{
		return this._MiscVars.Selection;
	},
	set Selection(aObject)
	{
		this._MiscVars.Selection = aObject;
	},
	get BMBCSSSheet()
	{
		return this._MiscVars.bmbCSSSheet;
	},
	HistoryGroupingStringToInt: function(aGroupMethod)
	{
		if (typeof(aGroupMethod) != 'string')
			return null;
		
		switch(aGroupMethod)
		{
			case "NC:HistoryRoot":
				return 0;
				
			case "NC:HistoryByDate":
				return 1;
				
			case "NC:HistoryByDateAndSite":
				return 2;
				
			default:
				return 2;
		}
	},
	HistoryGroupingIntToString: function(aGroupMethod)
	{
		if (typeof(aGroupMethod) != 'number')
			return null;
		
		switch(aGroupMethod)
		{
			case 0:
				return "NC:HistoryRoot";
				
			case 1:
				return "NC:HistoryByDate";
				
			case 2:
				return "NC:HistoryByDateAndSite";
				
			default:
				return "NC:HistoryByDateAndSite";
		}
	},
	rebuildHistory: function()
	{
		var history = document.getElementById("history-menu-button-history-popup");
		if (history && history.builder)
		{
			history.builder.rebuild();
			this.HistoryNeedsRebuild = false;
		}
	},
	//#######################################
	//########	/private functions and properties	########
	//########						########
	//#######################################
	
	init: function()
	{
		bmbextHistory.loadPrefs();
		
		bmbextHistory.hideGoMenu();
		bmbextHistory.findBMBCSSSheet();
		bmbextHistory.showHideDropmarker();
		var button = document.getElementById('history-menu-button');
		if (button)
		{
			bmbextHistory.checkForContextMenu();
			bmbextHistory.addBMBSettings();
			bmbextHistory.setHistoryGrouping();
			
			bmbextHistory._MiscVars.ShortcutKey = document.getElementById('go-menu').getAttribute("accesskey").charCodeAt(0);		//converts the character to the ASCII code because keyCode is a ASCII code
			window.addEventListener('keydown', bmbextHistory.catchKKey, false);
			window.addEventListener('keyup', bmbextHistory.catchKKey, false);
			
			bmbextHistory.setGlobalHistoryDB();
		}
	},
	
	loadPrefs: function()
	{
		//load user settings
		this.PrefsIFace = this.prefRoot;
		
		this._Prefs.hideDropmarker = this.PrefsIFace.getBoolPref("dropmarker.hide");
		this._Prefs.hideGoMenu = this.PrefsIFace.getBoolPref("gomenu.hide");
		this._Prefs.hideBMBS = this.PrefsIFace.getBoolPref("settings.hide");
		this._Prefs.MiddleClickOpens = this.PrefsIFace.getIntPref("settings.MiddleClickOpens");
		this._Prefs.RightClickOpens = this.PrefsIFace.getIntPref("settings.RightClickOpens");
		this._Prefs.GroupMethod = this.PrefsIFace.getIntPref("settings.GroupMethod");
		this._Prefs.MaxRecentHistory = this.PrefsIFace.getIntPref("settings.MaxRecentHistory");
	},
	
	hideGoMenu: function()
	{
		document.getElementById('go-menu').hidden = document.getElementById('history-menu-button') && this._Prefs.hideGoMenu;	// only when hmb is used an when pref is true
	},
	findBMBCSSSheet: function()
	{
		var CSSSheets = document.styleSheets;
		for (var i=0, l=CSSSheets.length; i<l; i++)
		{
			if (CSSSheets[i].href == "chrome://bmb-ext/skin/bmb-ext.css")
			{
				this.BMBCSSSheet.globalIndex = i;
				this.BMBCSSSheet.sheetObj = CSSSheets[i];
				break;
			}
		}
	},
	showHideDropmarker: function()
	{
		var sheet = this.BMBCSSSheet;
		if (sheet.globalIndex < 0)
		{
			return;
		}
		
		var lastRuleIndex = sheet.sheetObj.cssRules.length - 1;
		
		if (!sheet.dropmarkerHidden && this._Prefs.hideDropmarker)
		{
			sheet.sheetObj.insertRule("#history-menu-button .toolbarbutton-menu-dropmarker { display: none !important;}", lastRuleIndex + 1);
			sheet.dropmarkerHidden = true;
			return;
		}
		
		if (sheet.dropmarkerHidden && !this._Prefs.hideDropmarker)
		{
			var indexToRemove = -1;
			for (var i=lastRuleIndex; i>=0; i--)
			{
				if (sheet.sheetObj.cssRules[i].selectorText == "#history-menu-button .toolbarbutton-menu-dropmarker")
				{
					indexToRemove = i;
					break;
				}
			}
			
			if (indexToRemove >= 0)
			{
				sheet.sheetObj.deleteRule(indexToRemove);
				sheet.dropmarkerHidden = false;
			}
		}
	},
	
	checkForContextMenu: function ()
	{
		//remove/disable toolbar context menu, if needed
		var button = document.getElementById('history-menu-button');
		
		if (this._Prefs.RightClickOpens > 0)
		{
			button.setAttribute("context", "");
		}
		else
		{
			button.setAttribute("context", "toolbar-context-menu");
		}
	},
	
	onMouseOverButton: function(aEvent)
	{
		if (aEvent.target.id == 'history-menu-button')
		{
			this.checkForContextMenu();
			this.addBMBSettings();
			this.setHistoryGrouping();
			if (this.HistoryNeedsRebuild)
			{
				this.rebuildHistory();
			}
		}
	},
	onPopupShowing: function(aEvent)
	{
		if (aEvent.target.id == "history-menu-button-popup")
		{
			this.checkForTabUndo();
			this.updateTabUndoMenu();
		}
	},
	onButtonCommand: function(aEvent)
	{
		if (aEvent.target.parentNode && aEvent.target.parentNode.id == "history-menu-button-UndoMenuPopup")
		{
			return;
		}
		
		var url = aEvent.target.statusText;
		if (url)
		{
			this.HistoryNeedsRebuild = true;
			openUILink(url, aEvent, false, true);
		}
	},
	onButtonClick: function(aEvent, aNode)
	{
		this.catchMKey(aEvent);
		this.checkForMiddleClick(aNode, aEvent);
	},
	
	checkForMiddleClick: function(aNode, aEvent)
	{
		if (aEvent.button == 1)
		{
			var url = aEvent.target.statusText;
			if (url)
			{
				this.onButtonCommand(aEvent);
			}
		}
	},
	
	addBMBSettings: function()
	{
		// adds/removes BMB settings menuitem
		var settings = document.getElementById('hmb-settings');
		if (!this._Prefs.hideBMBS)
		{
			if (!settings)
			{
				settings = document.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "menuitem");
				settings.id = "hmb-settings";
				settings.setAttribute("label", this.StringBundle.getString("bmb-ext.menu-label"));
				settings.tooltipText = this.StringBundle.getString("bmb-ext.bmb-tooltip1");
				settings.setAttribute("oncommand", "bmbext.OpenSettings(1)");
				settings.setAttribute("class", "menuitem-iconic");
				var settingssep = document.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "menuseparator");
				settingssep.id = "hmb-settings-sep";
				
				var HMBpopup = document.getElementById('history-menu-button-popup');
				HMBpopup.insertBefore(settingssep,HMBpopup.firstChild);
				HMBpopup.insertBefore(settings, settingssep);
			}
		}
		else
		{
			if (settings)
			{
				var HMBpopup = document.getElementById('history-menu-button-popup');
				var toRemove = document.getElementById('hmb-settings');
				HMBpopup.removeChild(toRemove);
				toRemove = document.getElementById('hmb-settings-sep');
				HMBpopup.removeChild(toRemove);
			}
		}
	},
	
	setHistoryGrouping: function()
	{
		var HMBHpopup = document.getElementById('history-menu-button-history-popup');
		if (this.HistoryGroupingStringToInt(HMBHpopup.ref) != this._Prefs.GroupMethod)
		{
			HMBHpopup.ref = this.HistoryGroupingIntToString(this._Prefs.GroupMethod);
		}
	},
	
	catchMKey: function(aEvent)
	{
		if (aEvent.target.id != 'history-menu-button' || aEvent.currentTarget.disabled)
			return;
		
		var hmb = aEvent.target;
		switch(aEvent.button)
		{
			case 0:	//left button; uses oncommand
				break;
				
			case 1:	//middle button
				if (!hmb.open)
				{
					if (this._Prefs.MiddleClickOpens == 1)
						toggleSidebar('viewHistorySidebar');
					
					if (this._Prefs.MiddleClickOpens == 2)
						this.openLastClosedTab();
				}
				break;
				
			case 2:	//right button
				if (!hmb.open)
				{
					if (this._Prefs.RightClickOpens == 1)
						toggleSidebar('viewHistorySidebar');
					
					if (this._Prefs.RightClickOpens == 2)
						this.openLastClosedTab();
				}	
				break;
				
			default:	//unknown
				break;
		}
	},	
	catchKKey: function(aEvent) {
		//shortcut key event handler
		var hmb = document.getElementById('history-menu-button');
		if (!hmb || hmb.open)
		{
			this._MiscVars.ShortcutKeyPressed = false;
			return;
		}
		
		if (!bmbextHistory._MiscVars.ShortcutKey)
		{
			return;
		}
		
		if (aEvent.type== 'keydown' && !bmbextHistory._MiscVars.ShortcutKeyPressed)
		{
			if (aEvent.altKey && (aEvent.keyCode == bmbextHistory._MiscVars.ShortcutKey))
			{
				bmbextHistory._MiscVars.ShortcutKeyPressed = true;
			}
		}
		if (aEvent.type== 'keyup' && bmbextHistory._MiscVars.ShortcutKeyPressed)
		{
			if (aEvent.altKey && (aEvent.keyCode == bmbextHistory._MiscVars.ShortcutKey))
			{
				bmbextHistory._MiscVars.ShortcutKeyPressed = false;
				bmbextHistory.openMenu();
			}
		}
	},
	openMenu: function()
	{
		this.checkForContextMenu();
		this.addBMBSettings();
		this.setHistoryGrouping();
		if (this.HistoryNeedsRebuild)
		{
			this.rebuildHistory();
		}
		
		var hmb = document.getElementById('history-menu-button');
		document.getElementById('history-menu-button-popup').showPopup(hmb, -1, -1, "popup");
	},
	
	openURLIn: function(where)
	{
		var target = document.popupNode;
		if (!target)
			return;
		
		this.Selection = target;
		var isContainer = target.getAttribute('container');
		if (isContainer)
			return;
		
		var url = target.statusText;
		
		if (!this.checkURLSecurity(url))
			return;
		
		this.HistoryNeedsRebuild = true;
		openUILinkIn(url, where);
	},
	historyAddBookmarks: function()
	{
		var target = document.popupNode;
		if (!target)
			return;
		
		this.Selection = target;
		var url = target.statusText;
		var title = target.label;
		BookmarksUtils.addBookmark(url, title, undefined);
	},
	historyCopyLink: function()
	{
		var target = document.popupNode;
		if (!target)
			return;
		
		this.Selection = target;
		var url = target.statusText;
		var clipboard = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper );
		clipboard.copyString(url);
	},
	checkURLSecurity: function(aURL)
	{
		var uri = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService).newURI(aURL, null, null);
		
		if (uri.schemeIs("javascript") || uri.schemeIs("data"))
		{
			var strBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);
			var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
			
			var historyBundle = strBundleService.createBundle("chrome://global/locale/history/history.properties");
			var errorStr = historyBundle.GetStringFromName("load-js-data-url-error");
			
			var brandBundle = null;
			var brandStr = "";
			try
			{
				brandBundle = strBundleService.createBundle("chrome://branding/locale/brand.properties");
				brandStr = brandBundle.GetStringFromName("brandShortName");
			}
			catch(e)
			{
				brandBundle = strBundleService.createBundle("chrome://global/locale/brand.properties");
				brandStr = brandBundle.GetStringFromName("brandShortName");
			}
			
			promptService.alert(window, brandStr, errorStr);
			return false;
		}
		return true;
	},
	
	onHistoryButtonHidden: function()
	{
		setTimeout(bmbextHistory.destroyRecentHistoryMenuItems, 0, document.getElementById('history-menu-button-popup'));
	},
	constructRecentHistoryMenuItem: function(HistoryMenu, beforeItem, url, title, date)
	{
		var menuitem = document.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "menuitem");
		menuitem.setAttribute("statustext", url);
		menuitem.setAttribute("onmouseover", "bmbext.setStatus(this);");
		menuitem.setAttribute("tooltiptext", title);
		menuitem.setAttribute("label", title);
		menuitem.setAttribute("class", "menuitem-iconic bookmark-item");
		menuitem.setAttribute("value", date);
		HistoryMenu.insertBefore(menuitem, beforeItem);
		return menuitem;
	},
	destroyRecentHistoryMenuItems: function(HistoryMenu)
	{
		var startSeparator = document.getElementById("startHistorySeparator2");
		var endSeparator = document.getElementById("endHistorySeparator2");
		endSeparator.hidden = true;
		var destroy = false;
		for (var i = 0, l=HistoryMenu.childNodes.length; i < l; i++)
		{
			var item = HistoryMenu.childNodes[i];
			if (item == endSeparator)
				break;
			
			if (destroy)
			{
				i--;
				HistoryMenu.removeChild(item);
			}
			
			if (item == startSeparator)
				destroy = true;
		}
	},
	setGlobalHistoryDB: function()
	{
		var history = document.getElementById("hiddenHistoryTree");
		
		if (history.hidden)
		{
			var globalHistory = Components.classes["@mozilla.org/browser/global-history;2"].getService(Components.interfaces.nsIRDFDataSource);
			history.database.AddDataSource(globalHistory);
			history.hidden = false;
			if (!history.ref)
				history.ref = "NC:HistoryRoot";
		}
		history.view.rowCount;
	},
	updateRecentHistory: function(HistoryButton) 
	{
		var history = document.getElementById("hiddenHistoryTree");
		if (history.hidden)
		{
			this.setGlobalHistoryDB();
		}
		var count = history.view.rowCount;
		if (count == 0)
			return;
		
		var maxCount = this._Prefs.MaxRecentHistory;
		if (maxCount <= 0)
			return;
		
		if (count > maxCount)
			count = maxCount;
		
		if (!gRDF)
			var gRDF = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(Components.interfaces.nsIRDFService);
		
		var builder = history.builder.QueryInterface(Components.interfaces.nsIXULTreeBuilder);
		var beforeItem = document.getElementById("endHistorySeparator2");  
		var nameResource = gRDF.GetResource("http://home.netscape.com/NC-rdf#Name");
		var dateResource = gRDF.GetResource("http://home.netscape.com/NC-rdf#Date");
		var endSep = beforeItem;
		var showSep = false;
		for (var url, res, titleRes, dateRes, titleLiteral, dateLiteral, i = count-1; i >= 0; i--)
		{
			res = builder.getResourceAtIndex(i);
			url = res.Value;
			try
			{
				titleRes = history.database.GetTarget(res, nameResource, true);
				dateRes = history.database.GetTarget(res, dateResource, true);
			}
			catch(e)
			{
				continue;
			}
			
			showSep = true;
			titleLiteral = titleRes.QueryInterface(Components.interfaces.nsIRDFLiteral);
			dateLiteral = dateRes.QueryInterface(Components.interfaces.nsIRDFDate);
			beforeItem = this.constructRecentHistoryMenuItem(HistoryButton, beforeItem, url, titleLiteral.Value, dateLiteral.Value);
		}
		if (showSep)
			endSep.hidden = false;
	},
	
	buildHistoryContextMenu: function(aEvent)
	{
		var target = document.popupNode;
		if (!target)
			return;
		
		this.Selection = target;
		
		var openItem = document.getElementById("miOpen");
		var openItemInNewWindow = document.getElementById("miOpenInNewWindow");
		var openItemInNewTab = document.getElementById("miOpenInNewTab");
		var sep1 = document.getElementById("pre-bookmarks-separator");
		var bookmarkItem = document.getElementById("miAddBookmark");
		var copyLocationItem = document.getElementById("miCopyLink");
		var sep2 = document.getElementById("post-bookmarks-separator");
		var deleteItem = document.getElementById("cMenu_delete");
		
		var isContainer = target.getAttribute('container');
		var HistoryGrouping = this._Prefs.GroupMethod;
		if (isContainer && (HistoryGrouping == 2 || HistoryGrouping == 1))
		{
			openItem.hidden = true;
			openItemInNewWindow.hidden = true;
			openItemInNewTab.hidden = true;
			sep1.hidden = true;
			bookmarkItem.hidden = true;
			copyLocationItem.hidden = true;
			sep2.hidden = true;
			deleteItem.hidden = false;
		}
		else
		{
			openItem.hidden = false;
			openItemInNewWindow.hidden = false;
			openItemInNewTab.hidden = false;
			sep1.hidden = false;
			bookmarkItem.hidden = false;
			copyLocationItem.hidden = false;
			sep2.hidden = false;
			deleteItem.hidden = false;
		}
	},
	destroyHistoryContextMenu: function()
	{
		if (content)
			content.focus();
		
		closeMenus(bmbextHistory.Selection);
		bmbextHistory.Selection = null;
	},
	
	/*
		DELETE COMMAND
	*/
	doDelete: function()
	{
		setTimeout(bmbextHistory.doRealDelete, 1, document.popupNode);
	},
	doRealDelete: function(aNode)
	{
		var target = aNode;
		if (!target)
			return false;   
		
		var history = document.getElementById("history-menu-button-history-popup");
		var datasource = history.database;
		var dsEnum = datasource.GetDataSources();
		dsEnum.getNext();
		var ds = dsEnum.getNext().QueryInterface(Components.interfaces.nsIRDFDataSource);
		
		ds.beginUpdateBatch();
		
		var dirty = false;
		
		if (!gRDF)
			gRDF = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(Components.interfaces.nsIRDFService);
		
		var IDRes = target.resource;
		if (!IDRes)
			return false;	//continue;
		
		var root = history.getAttribute('ref');
		var parentIDRes;
		try
		{
			parentIDRes = target.parentNode.parentNode.resource;		//if target is a menuitem, than the parent would be a menupopup
		}
		catch(e)
		{
			parentIDRes = gRDF.GetResource(root);
		}
		if (!parentIDRes)
			return false;	//continue;
		
		// otherwise remove the parent/child assertion then
		var containment = gRDF.GetResource("http://home.netscape.com/NC-rdf#child");
		ds.Unassert(parentIDRes, containment, IDRes);
		dirty = true;
		
		ds.endUpdateBatch();
		
		if (dirty)
		{    
			try
			{
				var remote = datasource.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
				remote.Flush();
			}
			catch (e)
			{}
		}
		
		return true;
	},
	
	/*
		SANITIZE CODE	-	the clean and simple way
	*/
	updateSanitizeCommand: function()
	{
		var completeHistory = document.getElementById("history-menu-button-history-popup");
		var SanitizeHistoryItem = document.getElementById("SanitizeHistory");
		var active = false;	//state of item - false by default
		var globalHistory = Components.classes["@mozilla.org/browser/global-history;2"].getService(Components.interfaces.nsIBrowserHistory);
		
		active = (globalHistory.count > 0);
		
		SanitizeHistoryItem.setAttribute("disabled", !active);
		completeHistory.setAttribute("disabled", !active);
	},
	doSanitizeHistory: function()
	{
		setTimeout("bmbextHistory.doRealSanitizeHistory()", 1);
	},
	doRealSanitizeHistory: function()
	{
		var globalHistory = Components.classes["@mozilla.org/browser/global-history;2"].getService(Components.interfaces.nsIBrowserHistory);
		globalHistory.removeAllPages();
		
		try
		{
			var observer = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
			observer.notifyObservers(null, "browser:purge-session-history", "");
		}
		catch(e)
		{}
		
		this.HistoryNeedsRebuild = true;
	},
	
	/*
		UNDO MENU and extra functions
	*/
	checkForTabUndo: function()
	{
		if (!this._MiscVars.canUndoCloseTab)
			return;
		
		var menuUndo = document.getElementById("historyUndoMenu");
		if (menuUndo)
		{
			var buttonUndoMenu = document.getElementById("history-menu-button-UndoMenu");
			buttonUndoMenu.setAttribute("label", menuUndo.getAttribute("label"));
			buttonUndoMenu.previousSibling.hidden = false;
			buttonUndoMenu.hidden = false;
		}
	},
	updateTabUndoMenu: function()
	{
		var buttonUndoMenu = document.getElementById("history-menu-button-UndoMenu");
		if (!this._MiscVars.canUndoCloseTab || buttonUndoMenu.hidden)
		{
			return;
		}
		
		var undoPopup = document.getElementById("history-menu-button-UndoMenuPopup");
		// remove existing menu items
		while (undoPopup.hasChildNodes())
		{
			undoPopup.removeChild(undoPopup.firstChild);
		}
		
		// get closed-tabs from nsSessionStore
		var ss = Components.classes["@mozilla.org/browser/sessionstore;1"].getService(Components.interfaces.nsISessionStore);
		// no restorable tabs, so make sure menu is disabled, and return
		if (ss.getClosedTabCount(window) == 0)
		{
			undoPopup.parentNode.setAttribute("disabled", true);
			return;
		}
		
		// enable menu
		undoPopup.parentNode.removeAttribute("disabled");
		
		// populate menu
		var undoItems = eval("(" + ss.getClosedTabData(window) + ")");
		var menuitem;
		for (var i=0, l=undoItems.length, itemTitle, itemURL, imgsrc; i < l; i++)
		{
			itemTitle = undoItems[i].title;
			itemURL = undoItems[i].state.entries[undoItems[i].state.index - 1].url;
			menuitem = undoPopup.appendChild(document.createElement("menuitem"));
			menuitem.setAttribute("label", itemTitle);
			menuitem.setAttribute("oncommand", "undoCloseTab(" + i + ");");
			menuitem.setAttribute("tooltiptext", itemTitle);
			menuitem.setAttribute("statustext", itemURL);
			menuitem.setAttribute("onmouseover", "bmbext.setStatus(this);");
			menuitem.setAttribute("class", "menuitem-iconic tabitem");
			menuitem.setAttribute("onclick", "");
			imgsrc = this.getTabImage(undoItems[i]);
			if (imgsrc && !gBrowser.isIconKnownMissing(imgsrc))	// isIconKnownMissing() -> http://lxr.mozilla.org/mozilla1.8/source/toolkit/content/widgets/tabbrowser.xml#562
			{
				menuitem.setAttribute("image", imgsrc);
			}
		}
		
		// "open all in tabs"
		var bundleService = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);
		var bmStringBundle = bundleService.createBundle("chrome://browser/locale/bookmarks/bookmarks.properties");
		undoPopup.appendChild(document.createElement("menuseparator"));
		menuitem = undoPopup.appendChild(document.createElement("menuitem"));
		menuitem.setAttribute("label", bmStringBundle.GetStringFromName("cmd_bm_openfolder"));
		menuitem.setAttribute("accesskey", bmStringBundle.GetStringFromName("cmd_bm_openfolder_accesskey"));
		menuitem.setAttribute("oncommand", "bmbextHistory.openClosedTabs(" + undoItems.length + ");");
		menuitem.setAttribute("class", "menuitem-iconic openall");
		
		// "clear list"
		menuitem = undoPopup.appendChild(document.createElement("menuitem"));
		menuitem.setAttribute("label", this.StringBundle.getString("bmb-ext.cleartablist.label"));
		menuitem.setAttribute("accesskey", this.StringBundle.getString("bmb-ext.cleartablist.accesskey"));
		menuitem.setAttribute("oncommand", "bmbextHistory.clearClosedTabs();");
		menuitem.setAttribute("class", "menuitem-iconic cleartabs");
	},
	openClosedTabs: function(numTabs)
	{
		for (var i = 0; i<numTabs; i++)
		{
			undoCloseTab(0);
		}
	},
	openLastClosedTab: function()
	{
		if (this._MiscVars.canUndoCloseTab)
			undoCloseTab(0);
	},
	getTabImage: function(aTabObj)
	{
		var xultab = aTabObj.state.xultab.split(" ");
		var image;
		xultab.forEach
		(
			function(aValue)
			{
				if (aValue.search("image") == 0)
				{
					image = aValue.slice(aValue.search("=")+1);
				}
			}
		);
		return decodeURI(image);
	},
	clearClosedTabs: function()
	{
		var ss = Components.classes["@mozilla.org/browser/sessionstore;1"].getService(Components.interfaces.nsISessionStore);
		if (ss.getClosedTabCount(window) == 0)
		{
			return;
		}
		
		this.PrefsIFace = "browser.sessionstore.";
		try
		{
			const USER_MAX_TABS_UNDO = this.PrefsIFace.getIntPref("max_tabs_undo");
		}
		catch(e){}
		
		this.PrefsIFace.setIntPref("max_tabs_undo", 0);	// causes sessionstore to remove all stored tabs
		if (USER_MAX_TABS_UNDO != null)			// set the value back to the previous value
		{
			this.PrefsIFace.setIntPref("max_tabs_undo", USER_MAX_TABS_UNDO);
		}
		else
		{
			this.PrefsIFace.clearUserPref("max_tabs_undo");
		}
		this.PrefsIFace = this.prefRoot;
	}
}
 
window.addEventListener('load', bmbextHistory.init, false);